@@ -16,9 +16,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -26,11 +24,20 @@
 #endif
 
 #include "configuration.h"
-/* @todo the inclusion of server.h here is a layering violation */
-#include <server/server.h>
+#include "log.h"
+#include "command.h"
 
 #include <getopt.h>
 
+#include <limits.h>
+#include <stdlib.h>
+#if IS_DARWIN
+#include <libproc.h>
+#endif
+#ifdef HAVE_SYS_SYSCTL_H
+#include <sys/sysctl.h>
+#endif
+
 static int help_flag, version_flag;
 
 static const struct option long_options[] = {
@@ -52,52 +59,129 @@ int configuration_output_handler(struct command_context *context, const char *li
 	return ERROR_OK;
 }
 
-#ifdef _WIN32
-static char *find_suffix(const char *text, const char *suffix)
+/* Return the canonical path to the directory the openocd executable is in.
+ * The path should be absolute, use / as path separator and have all symlinks
+ * resolved. The returned string is malloc'd. */
+static char *find_exe_path(void)
 {
-	size_t text_len = strlen(text);
-	size_t suffix_len = strlen(suffix);
+	char *exepath = NULL;
 
-	if (suffix_len == 0)
-		return (char *)text + text_len;
+	do {
+#if IS_WIN32 && !IS_CYGWIN
+		exepath = malloc(MAX_PATH);
+		if (exepath == NULL)
+			break;
+		GetModuleFileName(NULL, exepath, MAX_PATH);
 
-	if (suffix_len > text_len || strncmp(text + text_len - suffix_len, suffix, suffix_len) != 0)
-		return NULL; /* Not a suffix of text */
+		/* Convert path separators to UNIX style, should work on Windows also. */
+		for (char *p = exepath; *p; p++) {
+			if (*p == '\\')
+				*p = '/';
+		}
 
-	return (char *)text + text_len - suffix_len;
-}
+#elif IS_DARWIN
+		exepath = malloc(PROC_PIDPATHINFO_MAXSIZE);
+		if (exepath == NULL)
+			break;
+		if (proc_pidpath(getpid(), exepath, PROC_PIDPATHINFO_MAXSIZE) <= 0) {
+			free(exepath);
+			exepath = NULL;
+		}
+
+#elif defined(CTL_KERN) && defined(KERN_PROC) && defined(KERN_PROC_PATHNAME) /* *BSD */
+#ifndef PATH_MAX
+#define PATH_MAX 1024
 #endif
+		char *path = malloc(PATH_MAX);
+		if (path == NULL)
+			break;
+		int mib[] = { CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1 };
+		size_t size = PATH_MAX;
+
+		if (sysctl(mib, (u_int)ARRAY_SIZE(mib), path, &size, NULL, 0) != 0)
+			break;
+
+#ifdef HAVE_REALPATH
+		exepath = realpath(path, NULL);
+		free(path);
+#else
+		exepath = path;
+#endif
+
+#elif defined(HAVE_REALPATH) /* Assume POSIX.1-2008 */
+		/* Try Unices in order of likelihood. */
+		exepath = realpath("/proc/self/exe", NULL); /* Linux/Cygwin */
+		if (exepath == NULL)
+			exepath = realpath("/proc/self/path/a.out", NULL); /* Solaris */
+		if (exepath == NULL)
+			exepath = realpath("/proc/curproc/file", NULL); /* FreeBSD (Should be covered above) */
+#endif
+	} while (0);
+
+	if (exepath != NULL) {
+		/* Strip executable file name, leaving path */
+		*strrchr(exepath, '/') = '\0';
+	} else {
+		LOG_WARNING("Could not determine executable path, using configured BINDIR.");
+		LOG_DEBUG("BINDIR = %s", BINDIR);
+#ifdef HAVE_REALPATH
+		exepath = realpath(BINDIR, NULL);
+#else
+		exepath = strdup(BINDIR);
+#endif
+	}
+
+	return exepath;
+}
+
+static char *find_relative_path(const char *from, const char *to)
+{
+	size_t i;
+
+	/* Skip common /-separated parts of from and to */
+	i = 0;
+	for (size_t n = 0; from[n] == to[n]; n++) {
+		if (from[n] == '\0') {
+			i = n;
+			break;
+		}
+		if (from[n] == '/')
+			i = n + 1;
+	}
+	from += i;
+	to += i;
+
+	/* Count number of /-separated non-empty parts of from */
+	i = 0;
+	while (from[0] != '\0') {
+		if (from[0] != '/')
+			i++;
+		char *next = strchr(from, '/');
+		if (next == NULL)
+			break;
+		from = next + 1;
+	}
+
+	/* Prepend that number of ../ in front of to */
+	char *relpath = malloc(i * 3 + strlen(to) + 1);
+	relpath[0] = '\0';
+	for (size_t n = 0; n < i; n++)
+		strcat(relpath, "../");
+	strcat(relpath, to);
+
+	return relpath;
+}
 
 static void add_default_dirs(void)
 {
-	const char *run_prefix;
 	char *path;
-
-#ifdef _WIN32
-	char strExePath[MAX_PATH];
-	GetModuleFileName(NULL, strExePath, MAX_PATH);
-
-	/* Strip executable file name, leaving path */
-	*strrchr(strExePath, '\\') = '\0';
-
-	/* Convert path separators to UNIX style, should work on Windows also. */
-	for (char *p = strExePath; *p; p++) {
-		if (*p == '\\')
-			*p = '/';
-	}
-
-	char *end_of_prefix = find_suffix(strExePath, BINDIR);
-	if (end_of_prefix != NULL)
-		*end_of_prefix = '\0';
-
-	run_prefix = strExePath;
-#else
-	run_prefix = "";
-#endif
+	char *exepath = find_exe_path();
+	char *bin2data = find_relative_path(BINDIR, PKGDATADIR);
 
 	LOG_DEBUG("bindir=%s", BINDIR);
 	LOG_DEBUG("pkgdatadir=%s", PKGDATADIR);
-	LOG_DEBUG("run_prefix=%s", run_prefix);
+	LOG_DEBUG("exepath=%s", exepath);
+	LOG_DEBUG("bin2data=%s", bin2data);
 
 	/*
 	 * The directory containing OpenOCD-supplied scripts should be
@@ -113,6 +197,12 @@ static void add_default_dirs(void)
 			free(path);
 		}
 	}
+
+	path = getenv("OPENOCD_SCRIPTS");
+
+	if (path)
+		add_script_search_dir(path);
+
 #ifdef _WIN32
 	const char *appdata = getenv("APPDATA");
 
@@ -125,23 +215,25 @@ static void add_default_dirs(void)
 	}
 #endif
 
-	path = alloc_printf("%s%s%s", run_prefix, PKGDATADIR, "/site");
+	path = alloc_printf("%s/%s/%s", exepath, bin2data, "site");
 	if (path) {
 		add_script_search_dir(path);
 		free(path);
 	}
 
-	path = alloc_printf("%s%s%s", run_prefix, PKGDATADIR, "/scripts");
+	path = alloc_printf("%s/%s/%s", exepath, bin2data, "scripts");
 	if (path) {
 		add_script_search_dir(path);
 		free(path);
 	}
+
+	free(exepath);
+	free(bin2data);
 }
 
 int parse_cmdline_args(struct command_context *cmd_ctx, int argc, char *argv[])
 {
 	int c;
-	char command_buffer[128];
 
 	while (1) {
 		/* getopt_long stores the option index here. */
@@ -164,24 +256,26 @@ int parse_cmdline_args(struct command_context *cmd_ctx, int argc, char *argv[])
 				break;
 			case 'f':		/* --file | -f */
 			{
-				snprintf(command_buffer, 128, "script {%s}", optarg);
-				add_config_command(command_buffer);
+				char *command = alloc_printf("script {%s}", optarg);
+				add_config_command(command);
+				free(command);
 				break;
 			}
 			case 's':		/* --search | -s */
 				add_script_search_dir(optarg);
 				break;
 			case 'd':		/* --debug | -d */
-				if (optarg)
-					snprintf(command_buffer, 128, "debug_level %s", optarg);
-				else
-					snprintf(command_buffer, 128, "debug_level 3");
-				command_run_line(cmd_ctx, command_buffer);
+			{
+				char *command = alloc_printf("debug_level %s", optarg ? optarg : "3");
+				command_run_line(cmd_ctx, command);
+				free(command);
 				break;
+			}
 			case 'l':		/* --log_output | -l */
 				if (optarg) {
-					snprintf(command_buffer, 128, "log_output %s", optarg);
-					command_run_line(cmd_ctx, command_buffer);
+					char *command = alloc_printf("log_output %s", optarg);
+					command_run_line(cmd_ctx, command);
+					free(command);
 				}
 				break;
 			case 'c':		/* --command | -c */
