@@ -14,9 +14,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -42,6 +40,7 @@ static void buspirate_path_move(int num_states, tap_state_t *path);
 static void buspirate_runtest(int num_cycles);
 static void buspirate_scan(bool ir_scan, enum scan_type type,
 	uint8_t *buffer, int scan_size, struct scan_command *command);
+static void buspirate_stableclocks(int num_cycles);
 
 #define CMD_UNKNOWN       0x00
 #define CMD_PORT_MODE     0x01
@@ -192,6 +191,10 @@ static int buspirate_execute_queue(void)
 			buspirate_tap_execute();
 			jtag_sleep(cmd->cmd.sleep->us);
 				break;
+		case JTAG_STABLECLOCKS:
+			DEBUG_JTAG_IO("stable clock %i cycles", cmd->cmd.stableclocks->num_cycles);
+			buspirate_stableclocks(cmd->cmd.stableclocks->num_cycles);
+				break;
 		default:
 			LOG_ERROR("BUG: unknown JTAG command type encountered");
 			exit(-1);
@@ -602,6 +605,16 @@ static void buspirate_scan(bool ir_scan, enum scan_type type,
 		buspirate_state_move();
 }
 
+static void buspirate_stableclocks(int num_cycles)
+{
+	int i;
+	int tms = (tap_get_state() == TAP_RESET ? 1 : 0);
+
+	buspirate_tap_make_space(0, num_cycles);
+
+	for (i = 0; i < num_cycles; i++)
+		buspirate_tap_append(tms, 0);
+}
 
 /************************* TAP related stuff **********/
 
