@@ -12,32 +12,10 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
-
-
-
-static __inline__ void interface_jtag_add_dr_out_core(struct jtag_tap *targettap,
-		int num_fields,
-		const int *num_bits,
-		const uint32_t *value,
-		enum tap_state end_state)
-{
-	/* synchronously do the operation here */
-}
-
-static __inline__ void interface_jtag_add_dr_out(struct jtag_tap *targettap,
-		int num_fields,
-		const int *num_bits,
-		const uint32_t *value,
-		enum tap_state end_state)
-{
-	/* synchronously do the operation here */
-}
-
 #define interface_jtag_add_callback(callback, in) callback(in)
 
-#define interface_jtag_add_callback4(callback, in, data1, data2, data3) jtag_set_error(callback(in, data1, data2, data3))
+#define interface_jtag_add_callback4(callback, in, data1, data2, data3) \
+	jtag_set_error(callback(in, data1, data2, data3))
