@@ -13,9 +13,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -159,13 +157,15 @@ static int virtex2_load(struct pld_device *pld_device, const char *filename)
 
 	jtag_add_tlr();
 
-	virtex2_set_instr(virtex2_info->tap, 0xc);	/* JSTART */
+	if (!(virtex2_info->no_jstart))
+		virtex2_set_instr(virtex2_info->tap, 0xc);	/* JSTART */
 	jtag_add_runtest(13, TAP_IDLE);
-	virtex2_set_instr(virtex2_info->tap, 0x3f);	/* BYPASS */
-	virtex2_set_instr(virtex2_info->tap, 0x3f);	/* BYPASS */
-	virtex2_set_instr(virtex2_info->tap, 0xc);	/* JSTART */
+	virtex2_set_instr(virtex2_info->tap, 0x3f);		/* BYPASS */
+	virtex2_set_instr(virtex2_info->tap, 0x3f);		/* BYPASS */
+	if (!(virtex2_info->no_jstart))
+		virtex2_set_instr(virtex2_info->tap, 0xc);	/* JSTART */
 	jtag_add_runtest(13, TAP_IDLE);
-	virtex2_set_instr(virtex2_info->tap, 0x3f);	/* BYPASS */
+	virtex2_set_instr(virtex2_info->tap, 0x3f);		/* BYPASS */
 	jtag_execute_queue();
 
 	return ERROR_OK;
@@ -212,6 +212,10 @@ PLD_DEVICE_COMMAND_HANDLER(virtex2_pld_device_command)
 	virtex2_info = malloc(sizeof(struct virtex2_pld_device));
 	virtex2_info->tap = tap;
 
+	virtex2_info->no_jstart = 0;
+	if (CMD_ARGC >= 3)
+		COMMAND_PARSE_NUMBER(int, CMD_ARGV[2], virtex2_info->no_jstart);
+
 	pld->driver_priv = virtex2_info;
 
 	return ERROR_OK;
