@@ -13,9 +13,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 /* The specification for SVF is available here:
@@ -216,13 +214,14 @@ static int svf_read_command_from_file(FILE *fd);
 static int svf_check_tdo(void);
 static int svf_add_check_para(uint8_t enabled, int buffer_offset, int bit_len);
 static int svf_run_command(struct command_context *cmd_ctx, char *cmd_str);
+static int svf_execute_tap(void);
 
 static FILE *svf_fd;
 static char *svf_read_line;
 static size_t svf_read_line_size;
 static char *svf_command_buffer;
 static size_t svf_command_buffer_size;
-static int svf_line_number = 1;
+static int svf_line_number;
 static int svf_getline(char **lineptr, size_t *n, FILE *stream);
 
 #define SVF_MAX_BUFFER_SIZE_TO_COMMIT   (1024 * 1024)
@@ -230,6 +229,7 @@ static uint8_t *svf_tdi_buffer, *svf_tdo_buffer, *svf_mask_buffer;
 static int svf_buffer_index, svf_buffer_size ;
 static int svf_quiet;
 static int svf_nil;
+static int svf_ignore_error;
 
 /* Targetting particular tap */
 static int svf_tap_is_specified;
@@ -241,10 +241,44 @@ static long svf_total_lines;
 static int svf_percentage;
 static int svf_last_printed_percentage = -1;
 
+/*
+ * macro is used to print the svf hex buffer at desired debug level
+ * DEBUG, INFO, ERROR, USER
+ */
+#define SVF_BUF_LOG(_lvl, _buf, _nbits, _desc)							\
+	svf_hexbuf_print(LOG_LVL_##_lvl ,  __FILE__, __LINE__, __func__, _buf, _nbits, _desc)
+
+static void svf_hexbuf_print(int dbg_lvl, const char *file, unsigned line,
+							 const char *function, const uint8_t *buf,
+							 int bit_len, const char *desc)
+{
+	int j, len = 0;
+	int byte_len = DIV_ROUND_UP(bit_len, 8);
+	int msbits = bit_len % 8;
+
+	/* allocate 2 bytes per hex digit */
+	char *prbuf = malloc((byte_len * 2) + 2 + 1);
+	if (!prbuf)
+		return;
+
+	/* print correct number of bytes, mask excess bits where applicable */
+	uint8_t msb = buf[byte_len - 1] & (msbits ? (1 << msbits) - 1 : 0xff);
+	len = sprintf(prbuf, msbits <= 4 ? "0x%01"PRIx8 : "0x%02"PRIx8, msb);
+	for (j = byte_len - 2; j >= 0; j--)
+		len += sprintf(prbuf + len, "%02"PRIx8, buf[j]);
+
+	log_printf_lf(dbg_lvl, file, line, function, "%8s = %s", desc ? desc : " ", prbuf);
+
+	free(prbuf);
+}
+
 static int svf_realloc_buffers(size_t len)
 {
 	void *ptr;
 
+	if (svf_execute_tap() != ERROR_OK)
+		return ERROR_FAIL;
+
 	ptr = realloc(svf_tdi_buffer, len);
 	if (!ptr)
 		return ERROR_FAIL;
@@ -287,20 +321,6 @@ static void svf_free_xxd_para(struct svf_xxr_para *para)
 	}
 }
 
-static unsigned svf_get_mask_u32(int bitlen)
-{
-	uint32_t bitmask;
-
-	if (bitlen < 0)
-		bitmask = 0;
-	else if (bitlen >= 32)
-		bitmask = 0xFFFFFFFF;
-	else
-		bitmask = (1 << bitlen) - 1;
-
-	return bitmask;
-}
-
 int svf_add_statemove(tap_state_t state_to)
 {
 	tap_state_t state_from = cmd_queue_cur_state;
@@ -341,7 +361,7 @@ COMMAND_HANDLER(handle_svf_command)
 #define SVF_MAX_NUM_OF_OPTIONS 5
 	int command_num = 0;
 	int ret = ERROR_OK;
-	long long time_measure_ms;
+	int64_t time_measure_ms;
 	int time_measure_s, time_measure_m;
 
 	/* use NULL to indicate a "plain" svf file which accounts for
@@ -356,6 +376,8 @@ COMMAND_HANDLER(handle_svf_command)
 	/* parse command line */
 	svf_quiet = 0;
 	svf_nil = 0;
+	svf_progress_enabled = 0;
+	svf_ignore_error = 0;
 	for (unsigned int i = 0; i < CMD_ARGC; i++) {
 		if (strcmp(CMD_ARGV[i], "-tap") == 0) {
 			tap = jtag_tap_by_string(CMD_ARGV[i+1]);
@@ -372,6 +394,9 @@ COMMAND_HANDLER(handle_svf_command)
 		else if ((strcmp(CMD_ARGV[i],
 				  "progress") == 0) || (strcmp(CMD_ARGV[i], "-progress") == 0))
 			svf_progress_enabled = 1;
+		else if ((strcmp(CMD_ARGV[i],
+				  "ignore_error") == 0) || (strcmp(CMD_ARGV[i], "-ignore_error") == 0))
+			svf_ignore_error = 1;
 		else {
 			svf_fd = fopen(CMD_ARGV[i], "r");
 			if (svf_fd == NULL) {
@@ -391,7 +416,7 @@ COMMAND_HANDLER(handle_svf_command)
 	time_measure_ms = timeval_ms();
 
 	/* init */
-	svf_line_number = 1;
+	svf_line_number = 0;
 	svf_command_buffer_size = 0;
 
 	svf_check_tdo_para_index = 0;
@@ -510,7 +535,7 @@ COMMAND_HANDLER(handle_svf_command)
 	time_measure_s %= 60;
 	if (time_measure_ms < 1000)
 		command_print(CMD_CTX,
-			"\r\nTime used: %dm%ds%lldms ",
+			"\r\nTime used: %dm%ds%" PRId64 "ms ",
 			time_measure_m,
 			time_measure_s,
 			time_measure_ms);
@@ -555,11 +580,14 @@ free_all:
 
 	if (ERROR_OK == ret)
 		command_print(CMD_CTX,
-			"svf file programmed successfully for %d commands",
-			command_num);
+			      "svf file programmed %s for %d commands with %d errors",
+			      (svf_ignore_error > 1) ? "unsuccessfully" : "successfully",
+			      command_num,
+			      (svf_ignore_error > 1) ? (svf_ignore_error - 1) : 0);
 	else
 		command_print(CMD_CTX, "svf file programmed failed");
 
+	svf_ignore_error = 0;
 	return ret;
 }
 
@@ -838,20 +866,16 @@ static int svf_check_tdo(void)
 		if ((svf_check_tdo_para[i].enabled)
 				&& buf_cmp_mask(&svf_tdi_buffer[index_var], &svf_tdo_buffer[index_var],
 				&svf_mask_buffer[index_var], len)) {
-			unsigned bitmask;
-			unsigned received, expected, tapmask;
-			bitmask = svf_get_mask_u32(svf_check_tdo_para[i].bit_len);
-
-			memcpy(&received, svf_tdi_buffer + index_var, sizeof(unsigned));
-			memcpy(&expected, svf_tdo_buffer + index_var, sizeof(unsigned));
-			memcpy(&tapmask, svf_mask_buffer + index_var, sizeof(unsigned));
 			LOG_ERROR("tdo check error at line %d",
 				svf_check_tdo_para[i].line_num);
-			LOG_ERROR("read = 0x%X, want = 0x%X, mask = 0x%X",
-				received & bitmask,
-				expected & bitmask,
-				tapmask & bitmask);
-			return ERROR_FAIL;
+			SVF_BUF_LOG(ERROR, &svf_tdi_buffer[index_var], len, "READ");
+			SVF_BUF_LOG(ERROR, &svf_tdo_buffer[index_var], len, "WANT");
+			SVF_BUF_LOG(ERROR, &svf_mask_buffer[index_var], len, "MASK");
+
+			if (svf_ignore_error == 0)
+				return ERROR_FAIL;
+			else
+				svf_ignore_error++;
 		}
 	}
 	svf_check_tdo_para_index = 0;
@@ -1009,6 +1033,19 @@ XXR_common:
 			}
 			i_tmp = xxr_para_tmp->len;
 			xxr_para_tmp->len = atoi(argus[1]);
+			/* If we are to enlarge the buffers, all parts of xxr_para_tmp
+			 * need to be freed */
+			if (i_tmp < xxr_para_tmp->len) {
+				free(xxr_para_tmp->tdi);
+				xxr_para_tmp->tdi = NULL;
+				free(xxr_para_tmp->tdo);
+				xxr_para_tmp->tdo = NULL;
+				free(xxr_para_tmp->mask);
+				xxr_para_tmp->mask = NULL;
+				free(xxr_para_tmp->smask);
+				xxr_para_tmp->smask = NULL;
+			}
+
 			LOG_DEBUG("\tlength = %d", xxr_para_tmp->len);
 			xxr_para_tmp->data_mask = 0;
 			for (i = 2; i < num_of_argu; i += 2) {
@@ -1045,8 +1082,7 @@ XXR_common:
 					LOG_ERROR("fail to parse hex value");
 					return ERROR_FAIL;
 				}
-				LOG_DEBUG("\t%s = 0x%X", argus[i],
-						(**(int **)pbuffer_tmp) & svf_get_mask_u32(xxr_para_tmp->len));
+				SVF_BUF_LOG(DEBUG, *pbuffer_tmp, xxr_para_tmp->len, argus[i]);
 			}
 			/* If a command changes the length of the last scan of the same type and the
 			 * MASK parameter is absent, */
@@ -1163,7 +1199,7 @@ XXR_common:
 					svf_add_check_para(0, svf_buffer_index, i);
 				field.num_bits = i;
 				field.out_value = &svf_tdi_buffer[svf_buffer_index];
-				field.in_value = &svf_tdi_buffer[svf_buffer_index];
+				field.in_value = (xxr_para_tmp->data_mask & XXR_TDO) ? &svf_tdi_buffer[svf_buffer_index] : NULL;
 				if (!svf_nil) {
 					/* NOTE:  doesn't use SVF-specified state paths */
 					jtag_add_plain_dr_scan(field.num_bits,
@@ -1253,7 +1289,7 @@ XXR_common:
 					svf_add_check_para(0, svf_buffer_index, i);
 				field.num_bits = i;
 				field.out_value = &svf_tdi_buffer[svf_buffer_index];
-				field.in_value = &svf_tdi_buffer[svf_buffer_index];
+				field.in_value = (xxr_para_tmp->data_mask & XXR_TDO) ? &svf_tdi_buffer[svf_buffer_index] : NULL;
 				if (!svf_nil) {
 					/* NOTE:  doesn't use SVF-specified state paths */
 					jtag_add_plain_ir_scan(field.num_bits,
@@ -1509,11 +1545,7 @@ XXR_common:
 
 			/* output debug info */
 			if ((SIR == command) || (SDR == command)) {
-				int read_value;
-				memcpy(&read_value, svf_tdi_buffer, sizeof(int));
-				/* in debug mode, data is from index 0 */
-				int read_mask = svf_get_mask_u32(svf_check_tdo_para[0].bit_len);
-				LOG_DEBUG("\tTDO read = 0x%X", read_value & read_mask);
+				SVF_BUF_LOG(DEBUG, svf_tdi_buffer, svf_check_tdo_para[0].bit_len, "TDO read");
 			}
 		}
 	} else {
@@ -1535,7 +1567,7 @@ static const struct command_registration svf_command_handlers[] = {
 		.handler = handle_svf_command,
 		.mode = COMMAND_EXEC,
 		.help = "Runs a SVF file.",
-		.usage = "svf [-tap device.tap] <file> [quiet] [nil] [progress]",
+		.usage = "svf [-tap device.tap] <file> [quiet] [nil] [progress] [ignore_error]",
 	},
 	COMMAND_REGISTRATION_DONE
 };
