@@ -16,10 +16,9 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -63,11 +62,10 @@ static int arm926ejs_cp15_read(struct target *target, uint32_t op1, uint32_t op2
 
 	buf_set_u32(address_buf, 0, 14, address);
 
-	if ((retval = arm_jtag_scann(jtag_info, 0xf, TAP_IDLE)) != ERROR_OK)
-	{
+	retval = arm_jtag_scann(jtag_info, 0xf, TAP_IDLE);
+	if (retval != ERROR_OK)
 		return retval;
-	}
-	retval = arm_jtag_set_instr(jtag_info, jtag_info->intest_instr, NULL, TAP_IDLE);
+	retval = arm_jtag_set_instr(jtag_info->tap, jtag_info->intest_instr, NULL, TAP_IDLE);
 	if (retval != ERROR_OK)
 		return retval;
 
@@ -89,10 +87,9 @@ static int arm926ejs_cp15_read(struct target *target, uint32_t op1, uint32_t op2
 
 	jtag_add_dr_scan(jtag_info->tap, 4, fields, TAP_IDLE);
 
-	long long then = timeval_ms();
+	int64_t then = timeval_ms();
 
-	for (;;)
-	{
+	for (;;) {
 		/* rescan with NOP, to wait for the access to complete */
 		access_t = 0;
 		nr_w_buf = 0;
@@ -100,19 +97,15 @@ static int arm926ejs_cp15_read(struct target *target, uint32_t op1, uint32_t op2
 
 		jtag_add_callback(arm_le_to_h_u32, (jtag_callback_data_t)value);
 
-		if ((retval = jtag_execute_queue()) != ERROR_OK)
-		{
+		retval = jtag_execute_queue();
+		if (retval != ERROR_OK)
 			return retval;
-		}
 
 		if (buf_get_u32(&access_t, 0, 1) == 1)
-		{
 			break;
-		}
 
 		/* 10ms timeout */
-		if ((timeval_ms()-then)>10)
-		{
+		if ((timeval_ms()-then) > 10) {
 			LOG_ERROR("cp15 read operation timed out");
 			return ERROR_FAIL;
 		}
@@ -122,7 +115,7 @@ static int arm926ejs_cp15_read(struct target *target, uint32_t op1, uint32_t op2
 	LOG_DEBUG("addr: 0x%x value: %8.8x", address, *value);
 #endif
 
-	retval = arm_jtag_set_instr(jtag_info, 0xc, NULL, TAP_IDLE);
+	retval = arm_jtag_set_instr(jtag_info->tap, 0xc, NULL, TAP_IDLE);
 	if (retval != ERROR_OK)
 		return retval;
 
@@ -155,11 +148,10 @@ static int arm926ejs_cp15_write(struct target *target, uint32_t op1, uint32_t op
 	buf_set_u32(address_buf, 0, 14, address);
 	buf_set_u32(value_buf, 0, 32, value);
 
-	if ((retval = arm_jtag_scann(jtag_info, 0xf, TAP_IDLE)) != ERROR_OK)
-	{
+	retval = arm_jtag_scann(jtag_info, 0xf, TAP_IDLE);
+	if (retval != ERROR_OK)
 		return retval;
-	}
-	retval = arm_jtag_set_instr(jtag_info, jtag_info->intest_instr, NULL, TAP_IDLE);
+	retval = arm_jtag_set_instr(jtag_info->tap, jtag_info->intest_instr, NULL, TAP_IDLE);
 	if (retval != ERROR_OK)
 		return retval;
 
@@ -181,27 +173,22 @@ static int arm926ejs_cp15_write(struct target *target, uint32_t op1, uint32_t op
 
 	jtag_add_dr_scan(jtag_info->tap, 4, fields, TAP_IDLE);
 
-	long long then = timeval_ms();
+	int64_t then = timeval_ms();
 
-	for (;;)
-	{
+	for (;;) {
 		/* rescan with NOP, to wait for the access to complete */
 		access_t = 0;
 		nr_w_buf = 0;
 		jtag_add_dr_scan(jtag_info->tap, 4, fields, TAP_IDLE);
-		if ((retval = jtag_execute_queue()) != ERROR_OK)
-		{
+		retval = jtag_execute_queue();
+		if (retval != ERROR_OK)
 			return retval;
-		}
 
 		if (buf_get_u32(&access_t, 0, 1) == 1)
-		{
 			break;
-		}
 
 		/* 10ms timeout */
-		if ((timeval_ms()-then)>10)
-		{
+		if ((timeval_ms()-then) > 10) {
 			LOG_ERROR("cp15 write operation timed out");
 			return ERROR_FAIL;
 		}
@@ -211,7 +198,7 @@ static int arm926ejs_cp15_write(struct target *target, uint32_t op1, uint32_t op
 	LOG_DEBUG("addr: 0x%x value: %8.8x", address, value);
 #endif
 
-	retval = arm_jtag_set_instr(jtag_info, 0xf, NULL, TAP_IDLE);
+	retval = arm_jtag_set_instr(jtag_info->tap, 0xf, NULL, TAP_IDLE);
 	if (retval != ERROR_OK)
 		return retval;
 
@@ -236,14 +223,14 @@ static int arm926ejs_examine_debug_reason(struct target *target)
 	int retval;
 
 	embeddedice_read_reg(dbg_stat);
-	if ((retval = jtag_execute_queue()) != ERROR_OK)
+	retval = jtag_execute_queue();
+	if (retval != ERROR_OK)
 		return retval;
 
 	/* Method-Of-Entry (MOE) field */
 	debug_reason = buf_get_u32(dbg_stat->value, 6, 4);
 
-	switch (debug_reason)
-	{
+	switch (debug_reason) {
 		case 0:
 			LOG_DEBUG("no *NEW* debug entry (?missed one?)");
 			/* ... since last restart or debug reset ... */
@@ -337,7 +324,8 @@ static int arm926ejs_get_ttb(struct target *target, uint32_t *result)
 	int retval;
 	uint32_t ttb = 0x0;
 
-	if ((retval = arm926ejs->read_cp15(target, 0, 0, 2, 0, &ttb)) != ERROR_OK)
+	retval = arm926ejs->read_cp15(target, 0, 0, 2, 0, &ttb);
+	if (retval != ERROR_OK)
 		return retval;
 
 	*result = ttb;
@@ -360,8 +348,7 @@ static int arm926ejs_disable_mmu_caches(struct target *target, int mmu,
 	if (retval != ERROR_OK)
 		return retval;
 
-	if (mmu)
-	{
+	if (mmu) {
 		/* invalidate TLB */
 		retval = arm926ejs->write_cp15(target, 0, 0, 8, 7, 0x0);
 		if (retval != ERROR_OK)
@@ -370,8 +357,7 @@ static int arm926ejs_disable_mmu_caches(struct target *target, int mmu,
 		cp15_control &= ~0x1U;
 	}
 
-	if (d_u_cache)
-	{
+	if (d_u_cache) {
 		uint32_t debug_override;
 		/* read-modify-write CP15 debug override register
 		 * to enable "test and clean all" */
@@ -398,8 +384,7 @@ static int arm926ejs_disable_mmu_caches(struct target *target, int mmu,
 		cp15_control &= ~0x4U;
 	}
 
-	if (i_cache)
-	{
+	if (i_cache) {
 		/* invalidate ICache */
 		retval = arm926ejs->write_cp15(target, 0, 0, 7, 5, 0x0);
 		if (retval != ERROR_OK)
@@ -454,8 +439,7 @@ static int arm926ejs_post_debug_entry(struct target *target)
 		return retval;
 	LOG_DEBUG("cp15_control_reg: %8.8" PRIx32 "", arm926ejs->cp15_control_reg);
 
-	if (arm926ejs->armv4_5_mmu.armv4_5_cache.ctype == -1)
-	{
+	if (arm926ejs->armv4_5_mmu.armv4_5_cache.ctype == -1) {
 		uint32_t cache_type_reg;
 		/* identify caches */
 		retval = arm926ejs->read_cp15(target, 0, 1, 0, 0, &cache_type_reg);
@@ -530,15 +514,13 @@ static int arm926ejs_verify_pointer(struct command_context *cmd_ctx,
 /** Logs summary of ARM926 state for a halted target. */
 int arm926ejs_arch_state(struct target *target)
 {
-	static const char *state[] =
-	{
+	static const char *state[] = {
 		"disabled", "enabled"
 	};
 
 	struct arm926ejs_common *arm926ejs = target_to_arm926(target);
 
-	if (arm926ejs->common_magic != ARM926EJS_COMMON_MAGIC)
-	{
+	if (arm926ejs->common_magic != ARM926EJS_COMMON_MAGIC) {
 		LOG_ERROR("BUG: %s", arm926_not);
 		return ERROR_TARGET_INVALID;
 	}
@@ -557,40 +539,30 @@ int arm926ejs_soft_reset_halt(struct target *target)
 	int retval = ERROR_OK;
 	struct arm926ejs_common *arm926ejs = target_to_arm926(target);
 	struct arm7_9_common *arm7_9 = target_to_arm7_9(target);
-	struct arm *armv4_5 = &arm7_9->armv4_5_common;
+	struct arm *arm = &arm7_9->arm;
 	struct reg *dbg_stat = &arm7_9->eice_cache->reg_list[EICE_DBG_STAT];
 
-	if ((retval = target_halt(target)) != ERROR_OK)
-	{
+	retval = target_halt(target);
+	if (retval != ERROR_OK)
 		return retval;
-	}
 
-	long long then = timeval_ms();
+	int64_t then = timeval_ms();
 	int timeout;
-	while (!(timeout = ((timeval_ms()-then) > 1000)))
-	{
-		if (buf_get_u32(dbg_stat->value, EICE_DBG_STATUS_DBGACK, 1) == 0)
-		{
+	while (!(timeout = ((timeval_ms()-then) > 1000))) {
+		if (buf_get_u32(dbg_stat->value, EICE_DBG_STATUS_DBGACK, 1) == 0) {
 			embeddedice_read_reg(dbg_stat);
-			if ((retval = jtag_execute_queue()) != ERROR_OK)
-			{
+			retval = jtag_execute_queue();
+			if (retval != ERROR_OK)
 				return retval;
-			}
-		}  else
-		{
+		} else
 			break;
-		}
-		if (debug_level >= 1)
-		{
+		if (debug_level >= 1) {
 			/* do not eat all CPU, time out after 1 se*/
 			alive_sleep(100);
 		} else
-		{
 			keep_alive();
-		}
 	}
-	if (timeout)
-	{
+	if (timeout) {
 		LOG_ERROR("Failed to halt CPU after 1 sec");
 		return ERROR_TARGET_TIMEOUT;
 	}
@@ -600,16 +572,16 @@ int arm926ejs_soft_reset_halt(struct target *target)
 	/* SVC, ARM state, IRQ and FIQ disabled */
 	uint32_t cpsr;
 
-	cpsr = buf_get_u32(armv4_5->cpsr->value, 0, 32);
+	cpsr = buf_get_u32(arm->cpsr->value, 0, 32);
 	cpsr &= ~0xff;
 	cpsr |= 0xd3;
-	arm_set_cpsr(armv4_5, cpsr);
-	armv4_5->cpsr->dirty = 1;
+	arm_set_cpsr(arm, cpsr);
+	arm->cpsr->dirty = 1;
 
 	/* start fetching from 0x0 */
-	buf_set_u32(armv4_5->pc->value, 0, 32, 0x0);
-	armv4_5->pc->dirty = 1;
-	armv4_5->pc->valid = 1;
+	buf_set_u32(arm->pc->value, 0, 32, 0x0);
+	arm->pc->dirty = 1;
+	arm->pc->valid = 1;
 
 	retval = arm926ejs_disable_mmu_caches(target, 1, 1, 1);
 	if (retval != ERROR_OK)
@@ -636,13 +608,11 @@ int arm926ejs_write_memory(struct target *target, uint32_t address,
 	 * Also it should be moved to the callbacks that handle breakpoints
 	 * specifically and not the generic memory write fn's. See XScale code.
 	 **/
-	if (arm926ejs->armv4_5_mmu.mmu_enabled && (count == 1) && ((size==2) || (size==4)))
-	{
+	if (arm926ejs->armv4_5_mmu.mmu_enabled && (count == 1) && ((size == 2) || (size == 4))) {
 		/* special case the handling of single word writes to bypass MMU
 		 * to allow implementation of breakpoints in memory marked read only
 		 * by MMU */
-		if (arm926ejs->armv4_5_mmu.armv4_5_cache.d_u_cache_enabled)
-		{
+		if (arm926ejs->armv4_5_mmu.armv4_5_cache.d_u_cache_enabled) {
 			/* flush and invalidate data cache
 			 *
 			 * MCR p15,0,p,c7,c10,1 - clean cache line using virtual address
@@ -662,24 +632,20 @@ int arm926ejs_write_memory(struct target *target, uint32_t address,
 		retval = armv4_5_mmu_write_physical(target, &arm926ejs->armv4_5_mmu, pa, size, count, buffer);
 		if (retval != ERROR_OK)
 			return retval;
-	} else
-	{
-		if ((retval = arm7_9_write_memory(target, address, size, count, buffer)) != ERROR_OK)
+	} else {
+		retval = arm7_9_write_memory(target, address, size, count, buffer);
+		if (retval != ERROR_OK)
 			return retval;
 	}
 
 	/* If ICache is enabled, we have to invalidate affected ICache lines
 	 * the DCache is forced to write-through, so we don't have to clean it here
 	 */
-	if (arm926ejs->armv4_5_mmu.armv4_5_cache.i_cache_enabled)
-	{
-		if (count <= 1)
-		{
+	if (arm926ejs->armv4_5_mmu.armv4_5_cache.i_cache_enabled) {
+		if (count <= 1) {
 			/* invalidate ICache single entry with MVA */
 			arm926ejs->write_cp15(target, 0, 1, 7, 5, address);
-		}
-		else
-		{
+		} else {
 			/* invalidate ICache */
 			arm926ejs->write_cp15(target, 0, 0, 7, 5, address);
 		}
@@ -713,8 +679,8 @@ int arm926ejs_init_arch_info(struct target *target, struct arm926ejs_common *arm
 {
 	struct arm7_9_common *arm7_9 = &arm926ejs->arm7_9_common;
 
-	arm7_9->armv4_5_common.mrc = arm926ejs_mrc;
-	arm7_9->armv4_5_common.mcr = arm926ejs_mcr;
+	arm7_9->arm.mrc = arm926ejs_mrc;
+	arm7_9->arm.mcr = arm926ejs_mcr;
 
 	/* initialize arm7/arm9 specific info (including armv4_5) */
 	arm9tdmi_init_arch_info(target, arm7_9, tap);
@@ -723,6 +689,7 @@ int arm926ejs_init_arch_info(struct target *target, struct arm926ejs_common *arm
 
 	arm7_9->post_debug_entry = arm926ejs_post_debug_entry;
 	arm7_9->pre_restore_context = arm926ejs_pre_restore_context;
+	arm7_9->write_memory = arm926ejs_write_memory;
 
 	arm926ejs->read_cp15 = arm926ejs_cp15_read;
 	arm926ejs->write_cp15 = arm926ejs_cp15_write;
@@ -748,7 +715,7 @@ int arm926ejs_init_arch_info(struct target *target, struct arm926ejs_common *arm
 
 static int arm926ejs_target_create(struct target *target, Jim_Interp *interp)
 {
-	struct arm926ejs_common *arm926ejs = calloc(1,sizeof(struct arm926ejs_common));
+	struct arm926ejs_common *arm926ejs = calloc(1, sizeof(struct arm926ejs_common));
 
 	/* ARM9EJ-S core always reports 0x1 in Capture-IR */
 	target->tap->ir_capture_mask = 0x0f;
@@ -787,8 +754,7 @@ static int arm926ejs_mmu(struct target *target, int *enabled)
 {
 	struct arm926ejs_common *arm926ejs = target_to_arm926(target);
 
-	if (target->state != TARGET_HALTED)
-	{
+	if (target->state != TARGET_HALTED) {
 		LOG_ERROR("Target not halted");
 		return ERROR_TARGET_INVALID;
 	}
@@ -801,6 +767,7 @@ static const struct command_registration arm926ejs_exec_command_handlers[] = {
 		.name = "cache_info",
 		.handler = arm926ejs_handle_cache_info_command,
 		.mode = COMMAND_EXEC,
+		.usage = "",
 		.help = "display information about target caches",
 
 	},
@@ -814,14 +781,14 @@ const struct command_registration arm926ejs_command_handlers[] = {
 		.name = "arm926ejs",
 		.mode = COMMAND_ANY,
 		.help = "arm926ejs command group",
+		.usage = "",
 		.chain = arm926ejs_exec_command_handlers,
 	},
 	COMMAND_REGISTRATION_DONE
 };
 
 /** Holds methods for ARM926 targets. */
-struct target_type arm926ejs_target =
-{
+struct target_type arm926ejs_target = {
 	.name = "arm926ejs",
 
 	.poll = arm7_9_poll,
@@ -840,8 +807,7 @@ struct target_type arm926ejs_target =
 	.get_gdb_reg_list = arm_get_gdb_reg_list,
 
 	.read_memory = arm7_9_read_memory,
-	.write_memory = arm926ejs_write_memory,
-	.bulk_write_memory = arm7_9_bulk_write_memory,
+	.write_memory = arm7_9_write_memory_opt,
 
 	.checksum_memory = arm_checksum_memory,
 	.blank_check_memory = arm_blank_check_memory,
