@@ -13,9 +13,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -67,7 +65,7 @@ struct etm_reg_info {
 	uint8_t size;			/* low-N of 32 bits */
 	uint8_t mode;			/* RO, WO, RW */
 	uint8_t bcd_vers;		/* 1.0, 2.0, etc */
-	char *name;
+	const char *name;
 };
 
 /*
@@ -318,7 +316,7 @@ struct reg_cache *etm_build_reg_cache(struct target *target,
 		etm_core, 1);
 
 	etm_get_reg(reg_list);
-	etm_ctx->config = buf_get_u32(&arch_info->value, 0, 32);
+	etm_ctx->config = buf_get_u32(arch_info->value, 0, 32);
 	config = etm_ctx->config;
 
 	/* figure ETM version then add base registers */
@@ -334,7 +332,7 @@ struct reg_cache *etm_build_reg_cache(struct target *target,
 			etm_core + 1, 1);
 		etm_get_reg(reg_list + 1);
 		etm_ctx->id = buf_get_u32(
-				&arch_info[1].value, 0, 32);
+				arch_info[1].value, 0, 32);
 		LOG_DEBUG("ETM ID: %08x", (unsigned) etm_ctx->id);
 		bcd_vers = 0x10 + (((etm_ctx->id) >> 4) & 0xff);
 
@@ -515,7 +513,7 @@ static int etm_read_reg_w_check(struct reg *reg,
 	retval = arm_jtag_scann(etm_reg->jtag_info, 0x6, TAP_IDLE);
 	if (retval != ERROR_OK)
 		return retval;
-	retval = arm_jtag_set_instr(etm_reg->jtag_info,
+	retval = arm_jtag_set_instr(etm_reg->jtag_info->tap,
 			etm_reg->jtag_info->intest_instr,
 			NULL,
 			TAP_IDLE);
@@ -602,7 +600,7 @@ static int etm_write_reg(struct reg *reg, uint32_t value)
 	retval = arm_jtag_scann(etm_reg->jtag_info, 0x6, TAP_IDLE);
 	if (retval != ERROR_OK)
 		return retval;
-	retval = arm_jtag_set_instr(etm_reg->jtag_info,
+	retval = arm_jtag_set_instr(etm_reg->jtag_info->tap,
 			etm_reg->jtag_info->intest_instr,
 			NULL,
 			TAP_IDLE);
@@ -1701,7 +1699,7 @@ COMMAND_HANDLER(handle_etm_image_command)
 
 COMMAND_HANDLER(handle_etm_dump_command)
 {
-	struct fileio file;
+	struct fileio *file;
 	struct target *target;
 	struct arm *arm;
 	struct etm_context *etm_ctx;
@@ -1741,24 +1739,24 @@ COMMAND_HANDLER(handle_etm_dump_command)
 	if (fileio_open(&file, CMD_ARGV[0], FILEIO_WRITE, FILEIO_BINARY) != ERROR_OK)
 		return ERROR_FAIL;
 
-	fileio_write_u32(&file, etm_ctx->capture_status);
-	fileio_write_u32(&file, etm_ctx->control);
-	fileio_write_u32(&file, etm_ctx->trace_depth);
+	fileio_write_u32(file, etm_ctx->capture_status);
+	fileio_write_u32(file, etm_ctx->control);
+	fileio_write_u32(file, etm_ctx->trace_depth);
 
 	for (i = 0; i < etm_ctx->trace_depth; i++) {
-		fileio_write_u32(&file, etm_ctx->trace_data[i].pipestat);
-		fileio_write_u32(&file, etm_ctx->trace_data[i].packet);
-		fileio_write_u32(&file, etm_ctx->trace_data[i].flags);
+		fileio_write_u32(file, etm_ctx->trace_data[i].pipestat);
+		fileio_write_u32(file, etm_ctx->trace_data[i].packet);
+		fileio_write_u32(file, etm_ctx->trace_data[i].flags);
 	}
 
-	fileio_close(&file);
+	fileio_close(file);
 
 	return ERROR_OK;
 }
 
 COMMAND_HANDLER(handle_etm_load_command)
 {
-	struct fileio file;
+	struct fileio *file;
 	struct target *target;
 	struct arm *arm;
 	struct etm_context *etm_ctx;
@@ -1788,16 +1786,16 @@ COMMAND_HANDLER(handle_etm_load_command)
 	if (fileio_open(&file, CMD_ARGV[0], FILEIO_READ, FILEIO_BINARY) != ERROR_OK)
 		return ERROR_FAIL;
 
-	int filesize;
-	int retval = fileio_size(&file, &filesize);
+	size_t filesize;
+	int retval = fileio_size(file, &filesize);
 	if (retval != ERROR_OK) {
-		fileio_close(&file);
+		fileio_close(file);
 		return retval;
 	}
 
 	if (filesize % 4) {
 		command_print(CMD_CTX, "size isn't a multiple of 4, no valid trace data");
-		fileio_close(&file);
+		fileio_close(file);
 		return ERROR_FAIL;
 	}
 
@@ -1808,28 +1806,28 @@ COMMAND_HANDLER(handle_etm_load_command)
 
 	{
 		uint32_t tmp;
-		fileio_read_u32(&file, &tmp); etm_ctx->capture_status = tmp;
-		fileio_read_u32(&file, &tmp); etm_ctx->control = tmp;
-		fileio_read_u32(&file, &etm_ctx->trace_depth);
+		fileio_read_u32(file, &tmp); etm_ctx->capture_status = tmp;
+		fileio_read_u32(file, &tmp); etm_ctx->control = tmp;
+		fileio_read_u32(file, &etm_ctx->trace_depth);
 	}
 	etm_ctx->trace_data = malloc(sizeof(struct etmv1_trace_data) * etm_ctx->trace_depth);
 	if (etm_ctx->trace_data == NULL) {
 		command_print(CMD_CTX, "not enough memory to perform operation");
-		fileio_close(&file);
+		fileio_close(file);
 		return ERROR_FAIL;
 	}
 
 	for (i = 0; i < etm_ctx->trace_depth; i++) {
 		uint32_t pipestat, packet, flags;
-		fileio_read_u32(&file, &pipestat);
-		fileio_read_u32(&file, &packet);
-		fileio_read_u32(&file, &flags);
+		fileio_read_u32(file, &pipestat);
+		fileio_read_u32(file, &packet);
+		fileio_read_u32(file, &flags);
 		etm_ctx->trace_data[i].pipestat = pipestat & 0xff;
 		etm_ctx->trace_data[i].packet = packet & 0xffff;
 		etm_ctx->trace_data[i].flags = flags;
 	}
 
-	fileio_close(&file);
+	fileio_close(file);
 
 	return ERROR_OK;
 }
