@@ -18,9 +18,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
- *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -75,11 +73,6 @@ static int adapter_load_core_reg_u32(struct target *target,
 		LOG_DEBUG("load from core reg %i  value 0x%" PRIx32 "", (int)num, *value);
 		break;
 
-	case ARMV7M_FPSID:
-	case ARMV7M_FPEXC:
-		*value = 0;
-		break;
-
 	case ARMV7M_FPSCR:
 		/* Floating-point Status and Registers */
 		retval = target_write_u32(target, ARMV7M_SCS_DCRSR, 33);
@@ -88,7 +81,7 @@ static int adapter_load_core_reg_u32(struct target *target,
 		retval = target_read_u32(target, ARMV7M_SCS_DCRDR, value);
 		if (retval != ERROR_OK)
 			return retval;
-		LOG_DEBUG("load from core reg %i  value 0x%" PRIx32 "", (int)num, *value);
+		LOG_DEBUG("load from FPSCR  value 0x%" PRIx32, *value);
 		break;
 
 	case ARMV7M_S0 ... ARMV7M_S31:
@@ -99,11 +92,8 @@ static int adapter_load_core_reg_u32(struct target *target,
 		retval = target_read_u32(target, ARMV7M_SCS_DCRDR, value);
 		if (retval != ERROR_OK)
 			return retval;
-		LOG_DEBUG("load from core reg %i  value 0x%" PRIx32 "", (int)num, *value);
-		break;
-
-	case ARMV7M_D0 ... ARMV7M_D15:
-		value = 0;
+		LOG_DEBUG("load from FPU reg S%d  value 0x%" PRIx32,
+			  (int)(num - ARMV7M_S0), *value);
 		break;
 
 	case ARMV7M_PRIMASK:
@@ -176,10 +166,6 @@ static int adapter_store_core_reg_u32(struct target *target,
 		LOG_DEBUG("write core reg %i value 0x%" PRIx32 "", (int)num, value);
 		break;
 
-	case ARMV7M_FPSID:
-	case ARMV7M_FPEXC:
-		break;
-
 	case ARMV7M_FPSCR:
 		/* Floating-point Status and Registers */
 		retval = target_write_u32(target, ARMV7M_SCS_DCRDR, value);
@@ -188,7 +174,7 @@ static int adapter_store_core_reg_u32(struct target *target,
 		retval = target_write_u32(target, ARMV7M_SCS_DCRSR, 33 | (1<<16));
 		if (retval != ERROR_OK)
 			return retval;
-		LOG_DEBUG("write core reg %i value 0x%" PRIx32 "", (int)num, value);
+		LOG_DEBUG("write FPSCR value 0x%" PRIx32, value);
 		break;
 
 	case ARMV7M_S0 ... ARMV7M_S31:
@@ -199,10 +185,8 @@ static int adapter_store_core_reg_u32(struct target *target,
 		retval = target_write_u32(target, ARMV7M_SCS_DCRSR, (num-ARMV7M_S0+64) | (1<<16));
 		if (retval != ERROR_OK)
 			return retval;
-		LOG_DEBUG("write core reg %i value 0x%" PRIx32 "", (int)num, value);
-		break;
-
-	case ARMV7M_D0 ... ARMV7M_D15:
+		LOG_DEBUG("write FPU reg S%d  value 0x%" PRIx32,
+			  (int)(num - ARMV7M_S0), value);
 		break;
 
 	case ARMV7M_PRIMASK:
@@ -357,7 +341,7 @@ static int adapter_init_target(struct command_context *cmd_ctx,
 	LOG_DEBUG("%s", __func__);
 
 	armv7m_build_reg_cache(target);
-
+	arm_semihosting_init(target);
 	return ERROR_OK;
 }
 
@@ -443,7 +427,7 @@ static int adapter_debug_entry(struct target *target)
 
 	LOG_DEBUG("entered debug state in core mode: %s at PC 0x%08" PRIx32 ", target->state: %s",
 		arm_mode_name(arm->core_mode),
-		*(uint32_t *)(arm->pc->value),
+		buf_get_u32(arm->pc->value, 0, 32),
 		target_state_name(target));
 
 	return retval;
@@ -786,6 +770,9 @@ static const struct command_registration adapter_command_handlers[] = {
 	{
 		.chain = arm_command_handlers,
 	},
+	{
+		.chain = armv7m_trace_command_handlers,
+	},
 	COMMAND_REGISTRATION_DONE
 };
 
@@ -794,6 +781,7 @@ struct target_type hla_target = {
 	.deprecated_name = "stm32_stlink",
 
 	.init_target = adapter_init_target,
+	.deinit_target = cortex_m_deinit_target,
 	.target_create = adapter_target_create,
 	.examine = cortex_m_examine,
 	.commands = adapter_command_handlers,
