@@ -63,9 +63,9 @@ proc ocd_process_reset_inner { MODE } {
 
 	# Examine all targets on enabled taps.
 	foreach t $targets {
-		if {[jtag tapisenabled [$t cget -chain-position]]} {
+		if {![using_jtag] || [jtag tapisenabled [$t cget -chain-position]]} {
 			$t invoke-event examine-start
-			set err [catch "$t arp_examine"]
+			set err [catch "$t arp_examine allow-defer"]
 			if { $err == 0 } {
 				$t invoke-event examine-end
 			}
@@ -79,7 +79,7 @@ proc ocd_process_reset_inner { MODE } {
 	}
 	foreach t $targets {
 		# C code needs to know if we expect to 'halt'
-		if {[jtag tapisenabled [$t cget -chain-position]]} {
+		if {![using_jtag] || [jtag tapisenabled [$t cget -chain-position]]} {
 			$t arp_reset assert $halt
 		}
 	}
@@ -94,7 +94,7 @@ proc ocd_process_reset_inner { MODE } {
 	}
 	foreach t $targets {
 		# Again, de-assert code needs to know if we 'halt'
-		if {[jtag tapisenabled [$t cget -chain-position]]} {
+		if {![using_jtag] || [jtag tapisenabled [$t cget -chain-position]]} {
 			$t arp_reset deassert $halt
 		}
 	}
@@ -107,7 +107,13 @@ proc ocd_process_reset_inner { MODE } {
 	# first executing any instructions.
 	if { $halt } {
 		foreach t $targets {
-			if {[jtag tapisenabled [$t cget -chain-position]] == 0} {
+			if {[using_jtag] && ![jtag tapisenabled [$t cget -chain-position]]} {
+				continue
+			}
+
+			# don't wait for targets where examination is deferred
+			# they can not be halted anyway at this point
+			if { ![$t was_examined] && [$t examine_deferred] } {
 				continue
 			}
 
@@ -131,7 +137,13 @@ proc ocd_process_reset_inner { MODE } {
 	#Pass 2 - if needed "init"
 	if { 0 == [string compare init $MODE] } {
 		foreach t $targets {
-			if {[jtag tapisenabled [$t cget -chain-position]] == 0} {
+			if {[using_jtag] && ![jtag tapisenabled [$t cget -chain-position]]} {
+				continue
+			}
+
+			# don't wait for targets where examination is deferred
+			# they can not be halted anyway at this point
+			if { ![$t was_examined] && [$t examine_deferred] } {
 				continue
 			}
 
@@ -148,6 +160,21 @@ proc ocd_process_reset_inner { MODE } {
 	}
 }
 
+proc using_jtag {} {
+	set _TRANSPORT [ transport select ]
+	expr { [ string first "jtag" $_TRANSPORT ] != -1 }
+}
+
+proc using_swd {} {
+	set _TRANSPORT [ transport select ]
+	expr { [ string first "swd" $_TRANSPORT ] != -1 }
+}
+
+proc using_hla {} {
+	set _TRANSPORT [ transport select ]
+	expr { [ string first "hla" $_TRANSPORT ] != -1 }
+}
+
 #########
 
 # Temporary migration aid.  May be removed starting in January 2011.
